package com.winhc.max.compute.graph.job.enterprise_group;

import com.aliyun.odps.data.TableInfo;
import com.aliyun.odps.graph.GraphJob;
import com.aliyun.odps.graph.RemoveDuplicatesLoadingResolver;
import com.winhc.max.compute.graph.job.pagerank.CompanyComputingVertexResolver;
import com.winhc.max.compute.graph.util.ParameterTool;

import java.io.IOException;

/**
 * @author: XuJiakai
 * 2023/2/7 13:58
 */
public class EnterpriseGroupJob {

    public static void main(String[] args) throws IOException {
        ParameterTool parameterTool = ParameterTool.fromArgs(args);
        String numWorkers = parameterTool.getOrDefault("numWorkers", "70");
        String workerCPU = parameterTool.getOrDefault("workerCPU", "2");
        String workerMem = parameterTool.getOrDefault("workerMem", "8192");

        System.out.println("input args: " + String.join(" ", args));
        GraphJob job = new GraphJob();



        job.setMaxIteration(-1);
        job.setNumWorkers(Integer.parseInt(numWorkers));
        if(parameterTool.has("workerCPU")){
            job.setWorkerCPU(Integer.parseInt(workerCPU) * 100);
        }
        if(parameterTool.has("workerMem")){
            job.setWorkerMemory(Integer.parseInt(workerMem));
        }



        job.setGraphLoaderClass(EnterpriseGroupReader.class);
        job.setVertexClass(EnterpriseGroupVertex.class);


        job.setLoadingVertexResolver(RemoveDuplicatesLoadingResolver.class);
        job.setComputingVertexResolver(CompanyComputingVertexResolver.class);
        job.setAggregatorClass(EnterpriseGroupAggregator.class);


        job.addInput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName("tmp_xjk_enterprise_group_input_vertex")
                .build());
        job.addInput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName("tmp_xjk_enterprise_group_input_edge")
                .build());
        job.addOutput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName("tmp_xjk_enterprise_group_out")
                .build());


        long startTime = System.currentTimeMillis();
        job.run();
        System.out.println("Job Finished in "
                + (System.currentTimeMillis() - startTime) / 1000.0 + " seconds");


    }
}
