package com.winhc.max.compute.graph.job.enterprise_group;


import com.aliyun.odps.conf.Configuration;
import com.aliyun.odps.data.TableInfo;
import com.aliyun.odps.graph.Edge;
import com.aliyun.odps.graph.GraphLoader;
import com.aliyun.odps.graph.MutationContext;
import com.aliyun.odps.io.DoubleWritable;
import com.aliyun.odps.io.LongWritable;
import com.aliyun.odps.io.Text;
import com.aliyun.odps.io.WritableRecord;
import com.google.gson.Gson;
import com.winhc.max.compute.graph.job.enterprise_group.entity.EntGroupMsg;
import com.winhc.max.compute.graph.job.enterprise_group.entity.EnterpriseGroupVertexValue;
import com.winhc.max.compute.graph.job.enterprise_group.entity.HolderEdge;
import com.winhc.max.compute.graph.util.JsonUtils;
import com.winhc.max.compute.graph.util.WritableRecordExtensions;
import lombok.experimental.ExtensionMethod;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2023/2/7 11:12
 * <p>
 * 数据读入
 */
@ExtensionMethod({
        EnterpriseGroupUtils.class
        , JsonUtils.class
        , WritableRecordExtensions.class
})
public class EnterpriseGroupReader extends
        GraphLoader<Text, EnterpriseGroupVertexValue, HolderEdge, EntGroupMsg> {
    private Boolean isEdgeData;
    private Gson gson;

    @Override
    public void setup(Configuration conf, int workerId, TableInfo tableInfo, MutationContext<Text, EnterpriseGroupVertexValue, HolderEdge, EntGroupMsg> context) throws IOException {
        if (tableInfo != null) {
            isEdgeData = tableInfo.getTableName().endsWith("_edge");
        }
        gson = new Gson();
    }

    @Override
    public void load(LongWritable recordNum, WritableRecord record, MutationContext<Text, EnterpriseGroupVertexValue, HolderEdge, EntGroupMsg> context) throws IOException {
        if (isEdgeData) {
            //加载边
            Text companyId = record.getTextOrNull("company_id");
            Text holderKeyno = record.getTextOrNull("holder_keyno");
            Text holderName = record.getTextOrNull("holder_name");
            DoubleWritable investmentProportion = record.getOrNull("investment_proportion", DoubleWritable.class);
            if (investmentProportion == null) {
                return;
            }
            HolderEdge edgeValue = HolderEdge.of(holderKeyno.toString(), investmentProportion.get());

            Edge<Text, HolderEdge> edge = new Edge<Text, HolderEdge>(
                    holderKeyno, edgeValue);
            context.addEdgeRequest(companyId, edge);

            if (holderKeyno.toString().length() == 33) {
                //添加人的顶点
                EnterpriseGroupVertex enterpriseGroupVertex = new EnterpriseGroupVertex();
                enterpriseGroupVertex.setId(holderKeyno);
                enterpriseGroupVertex.setValue(EnterpriseGroupVertexValue.of(holderName.toString()));
                context.addVertexRequest(enterpriseGroupVertex);
            }

        } else {
            //加载点，个人的顶点也要算在内
            Text companyId = record.getTextOrNull("company_id");
            Text companyName = record.getTextOrNull("company_name");
            Text companyOrgTypeNew = record.getTextOrNull("company_org_type_new");
            Text legalEntity = record.getTextOrNull("legal_entity_ids");


            boolean partnership = false;

            List<String> companyOrgTypeNewList = gson.parseListStr(companyOrgTypeNew.toString());
            if (companyOrgTypeNewList != null)
                for (String tmpText : companyOrgTypeNewList) {
                    boolean flag = tmpText.contains("合伙");
                    if (flag) {
                        partnership = true;
                        break;
                    }
                }

            EnterpriseGroupVertex enterpriseGroupVertex = new EnterpriseGroupVertex();
            enterpriseGroupVertex.setId(companyId);
            if (partnership) {
                List<Map<String, String>> le = gson.parseListStrByMap(legalEntity.toString());

                List<String> list = le.stream().map(e -> {
                    if (e.get("deleted").equals("0")) {
                        return e.get("id");
                    } else {
                        return null;
                    }
                }).filter(Objects::nonNull).collect(Collectors.toList());
                enterpriseGroupVertex.setValue(EnterpriseGroupVertexValue.of(companyName.toString(), list));
            } else {
                enterpriseGroupVertex.setValue(EnterpriseGroupVertexValue.of(companyName.toString()));
            }
            context.addVertexRequest(enterpriseGroupVertex);
        }
    }
}
